@@ -40,6 +40,44 @@ namespace Rcpp{
 			return start + i ;
 		}
 		
+		Range& operator++() {
+		    start++ ; end_++ ;
+		    return *this ;
+		}
+		Range& operator++(int) {
+		    start++ ; end_++ ;
+		    return *this ;
+		}
+		
+		Range& operator--() {
+		    start-- ; end_-- ;
+		    return *this ;
+		}
+		Range& operator--(int) {
+		    start-- ; end_-- ;
+		    return *this ;
+		}
+		
+		Range& operator+=(int n) {
+		    start += n ; end_ += n ;
+		    return *this ;
+		}
+		
+		Range& operator-=(int n) {
+		    start -= n ; end_ -= n ;
+		    return *this ;
+		}
+		
+		Range operator+( int n ){
+		    return Range( start + n, end_ + n ) ;
+		}
+		Range operator-( int n ){
+		    return Range( start - n, end_ - n ) ;
+		}
+		
+		inline int get_start() const { return start ; }
+		inline int get_end() const { return end_ ; }
+		
 	private:
 		int start ;
 		int end_ ;
