@@ -644,7 +644,23 @@
 					_["expm1"] = expm1(xx)
 					) ;
 			'
-			) 
+			), 
+			"runit_sum" = list( 
+			   signature( x = "numeric" ), 
+			   '
+			   NumericVector xx(x) ;
+			   double res = sum( xx ) ;
+			   return wrap( res ) ;
+			   '
+			), 
+			"runit_cumsum" = list( 
+			   signature( x = "numeric" ), 
+			   '
+			   NumericVector xx(x) ;
+			   NumericVector res = cumsum( xx ) ;
+			   return res ;
+			   '
+			)
 		)
 		
 		signatures <- lapply( sugar.functions, "[[", 1L )
@@ -1257,3 +1273,19 @@ test.sugar.psigamma <- function(){
 		) )
 }
 
+test.sugar.sum <- function(){
+    fx <- .rcpp.sugar$runit_sum
+    x <- rnorm( 10 )
+    checkEquals( fx(x), sum(x) )
+    x[4] <- NA
+    checkEquals( fx(x), sum(x) )
+}
+
+test.sugar.cumsum <- function(){
+    fx <- .rcpp.sugar$runit_cumsum
+    x <- rnorm( 10 )
+    checkEquals( fx(x), cumsum(x) )
+    x[4] <- NA
+    checkEquals( fx(x), cumsum(x) )
+}
+
